/*
    (C) 1995-96 AROS - The Amiga Research OS
    (C) 2002 Sigbjrn "CISC" Skjret and Harry "Piru" Sintonen
    $Id: strtolong.c,v 1.3 2007/08/20 15:37:07 piru Exp $

    Desc: Convert a string into a long
    Lang: english
*/

#include "dos_intern.h"

#define DEBUG_STRTOLONG(x)		;

/*****************************************************************************

    NAME */
#include <proto/dos.h>

	AROS_LH2I(LONG, StrToLong,

/*  SYNOPSIS */
	AROS_LHA(STRPTR, string, D1),
	AROS_LHA(LONG *, value,  D2),

/*  LOCATION */
	struct DosLibrary *, DOSBase, 136, Dos)

/*  FUNCTION
	Convert a string to a long number.

    INPUTS
	string - The value to convert
	value - The result is returned here

    RESULT
	How many characters in the string were considered when it was
	converted or -1 if no valid number could be found.

    NOTES
	The routine doesn't check if the number if too large.

    EXAMPLE
	// Valid number are: 5, -1, +3, +0007, $feed, -0xBEEF, etc.

    BUGS

    SEE ALSO

    INTERNALS

    HISTORY
        04-07-02    Added ability to read hex-strings.

*****************************************************************************/
{
    AROS_LIBFUNC_INIT
    AROS_LIBBASE_EXT_DECL(struct DosLibrary *,DOSBase)

    LONG old_v, v = 0;
    STRPTR s = string;
    BOOL negative = FALSE;

    DEBUG_STRTOLONG(dprintf("StrToLong: string \"%s\" value 0x%lx\n", string, (ULONG) value));

    /* Skip leading whitespace characters */
    while (*s == ' ' || *s == '\t')
	++s;

    /* Swallow sign */
#ifdef AROS_DOS_COMPATIBLE
    if (*s == '-')
#else
    if (*s == '+' || *s == '-')
#endif
	negative = *s++ == '-';

    /* Is it a hex-string? */
    if (*s == '$' || (*s == '0' && (s[1] == 'x' || s[1] == 'X')))
    {
	LONG i = 0;

	/* Skip hex-identifier */
	if (*s++ != '$')
	    ++s;

	/* If there is no number return an error. */
	if ((*s < '0' || *s > '9') && (*s < 'a' || *s > 'f') && (*s < 'A' || *s > 'F'))
	{
	    *value = 0;

	    DEBUG_STRTOLONG(dprintf("StrToLong: no number, return error\n"));

	    /* Compatability return */
	    if (s[-1] == 'x' || s[-1] == 'X')
		return s - 1 - string;
	    else
		return -1;
	}

	/* Calculate result */
	do
	{
	    old_v = v;
	    v <<= 4;
	    if (negative && v < old_v)
		break;
	    old_v = v;
	    v += ((*s <= '9') ? (*s - '0') : ((*s <= 'F') ? (*s - 'A' + 10) : (*s - 'a' + 10)));

	    if (negative && v == -2147483648L)
	    {
		negative = FALSE;
		old_v = v;
		++s;
		break;
	    }
	    else if (negative && v < old_v)
		break;

	    ++s;

	    if (++i == 8) break;
	} while ((*s >= '0' && *s <= '9') || (*s >= 'a' && *s <= 'f') || (*s >= 'A' && *s <= 'F'));
    }
    else
    {
	/* If there is no number return an error. */
	if (*s < '0' || *s > '9')
	{
	    *value = 0;

	    DEBUG_STRTOLONG(dprintf("StrToLong: no number, return error\n"));

	    return -1;
	}

	/* Calculate result */
	do
	{
	    LONG x;
	    x = v;
	    old_v = v;
	    v <<= 3;
	    if (v < old_v)
		break;
	    old_v = v;
	    v += x;
	    if (v < old_v)
		break;
	    old_v = v;
	    v += x;
	    if (v < old_v)
		break;
	    old_v = v;
	    v = v + *s - '0';
	    if (negative && v == -2147483648L)
	    {
		negative = FALSE;
		old_v = v;
		++s;
		break;
	    }
	    else if (v < old_v)
		break;
	    ++s;
	} while (*s >= '0' && *s <= '9');
    }

    /* Overflow? */
    if (v < old_v)
	v = old_v;

    /* Negative? */
    if (negative)
	v = -v;

    /* All done. */
    *value = v;

    DEBUG_STRTOLONG(dprintf("StrToLong: *value = %ld return %ld\n", v, (LONG) (s - string)));

    return s - string;

    AROS_LIBFUNC_EXIT
} /* StrToLong */
